@@ -20,7 +20,7 @@ from typing import Optional, Union
 from lava.exceptions import MesaCITimeoutError
 from lava.utils.console_format import CONSOLE_LOG
 from lava.utils.gitlab_section import GitlabSection
-from lava.utils.lava_farm import LavaFarm, get_lava_farm
+from lava.utils.lava_farm import get_lava_farm
 from lava.utils.lava_log_hints import LAVALogHints
 from lava.utils.log_section import (
     DEFAULT_GITLAB_SECTION_TIMEOUTS,
@@ -43,7 +43,7 @@ class LogFollower:
     fallback_timeout: timedelta = FALLBACK_GITLAB_SECTION_TIMEOUT
     _buffer: list[str] = field(default_factory=list, init=False)
     log_hints: LAVALogHints = field(init=False)
-    lava_farm: LavaFarm = field(init=False, default=get_lava_farm())
+    lava_farm: str = field(init=False, default=get_lava_farm())
     _merge_next_line: str = field(default_factory=str, init=False)
 
     def __post_init__(self):
@@ -261,7 +261,9 @@ class LogFollower:
         elif line["lvl"] == "input":
             prefix = "$ "
             suffix = ""
-        elif line["lvl"] == "target" and self.lava_farm != LavaFarm.COLLABORA:
+        elif line["lvl"] == "target" and self.lava_farm != "collabora":
+            if self.lava_farm == "lima":
+                fix_lava_color_log(line)
             # gl_section_fix_gen will output the stored line if it can't find a
             # match for the first split line
             # So we can recover it and put it back to the buffer
@@ -270,6 +272,31 @@ class LogFollower:
 
         return f'{prefix}{line["msg"]}{suffix}'
 
+
+def fix_lava_color_log(line):
+    """This function is a temporary solution for the color escape codes mangling problem. There is
+    some problem in message passing between the LAVA dispatcher and the device under test (DUT).
+    Here \x1b or \\e character is missing before `[:digit::digit:?m` ANSI TTY color codes.
+    When this problem is fixed on the LAVA side, one should remove this function.
+
+    For example, instead of receiving "\x1b[31m" (red text), we receive "[31m".
+
+    The function fixes three types of mangled ANSI sequences:
+    1. Standard color codes like [31m → \x1b[31m
+    2. Line erase codes [0K → \x1b[0K
+    3. Specific color formatting codes with carriage return [0;3xm → \r\x1b[0;3xm
+
+    Note: most LAVA farms don't have this problem, except for Lima, which uses
+    an older version of LAVA.
+    """
+    # Fix standard ANSI color codes (e.g., [31m → \x1b[31m)
+    line["msg"] = re.sub(r"(\[\d{1,2}m)", "\x1b" + r"\1", line["msg"])
+    # Fix ANSI line erase codes (e.g., [0K → \x1b[0K)
+    line["msg"] = re.sub(r"(\[0K)", "\x1b" + r"\1", line["msg"])
+    # Fix ANSI color codes with formatting and carriage return (e.g., [0;31m → \r\x1b[0;31m)
+    line["msg"] = re.sub(r"(\[0;3\d{1,2}m)", "\r\x1b" + r"\1", line["msg"])
+
+
 def fix_lava_gitlab_section_log():
     """This function is a temporary solution for the Gitlab section markers
     splitting problem. Gitlab parses the following lines to define a collapsible
@@ -305,8 +332,12 @@ def fix_lava_gitlab_section_log():
             yield first_line
 
 
-
 def print_log(msg: str, *args) -> None:
+    is_section_header = msg.startswith("\x1b[0Ksection_")
+    if is_section_header:
+        print(msg, *args)
+        return
+
     # Reset color from timestamp, since `msg` can tint the terminal color
     ts = datetime.now(tz=UTC)
     ts_str = f"{ts.hour:02}:{ts.minute:02}:{ts.second:02}.{int(ts.microsecond / 1000):03}"
