@@ -287,15 +287,15 @@ CHIPSET(0xe221, bmg, "BMG G31", "Intel(R) Graphics")
 CHIPSET(0xe222, bmg, "BMG G31", "Intel(R) Graphics")
 CHIPSET(0xe223, bmg, "BMG G31", "Intel(R) Graphics")
 
-CHIPSET(0xb080, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb081, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb082, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb083, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb084, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb085, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb086, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb087, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb08f, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb090, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb0a0, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
-CHIPSET(0xb0b0, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb080, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb081, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb082, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb083, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb084, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb085, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb086, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb087, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb08f, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb090, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb0a0, ptl, "PTL", "Intel(R) Graphics")
+CHIPSET(0xb0b0, ptl, "PTL", "Intel(R) Graphics")
