@@ -59,7 +59,13 @@ optimize(nir_shader *nir)
       NIR_PASS(progress, nir, nir_opt_dce);
       NIR_PASS(progress, nir, nir_opt_dead_cf);
       NIR_PASS(progress, nir, nir_opt_cse);
-      NIR_PASS(progress, nir, nir_opt_peephole_select, 64, false, true);
+
+      nir_opt_peephole_select_options peephole_select_options = {
+         .limit = 64,
+         .expensive_alu_ok = true,
+      };
+      NIR_PASS(progress, nir, nir_opt_peephole_select,
+               &peephole_select_options);
       NIR_PASS(progress, nir, nir_opt_phi_precision);
       NIR_PASS(progress, nir, nir_opt_algebraic);
       NIR_PASS(progress, nir, nir_opt_constant_folding);
@@ -214,6 +220,9 @@ gather_atomic_info(nir_builder *b, nir_intrinsic_instr *intr, void *data)
 static const char *
 remap_variant(nir_function *func, unsigned variant, const char *target)
 {
+   /* pass_flags is only 32-bit */
+   assert(variant < 32 && "maximum # of variants");
+
    bool has_atomic = func->pass_flags & BITFIELD_BIT(variant);
 
    if (!has_atomic && !strcmp(target, "g13x"))
@@ -286,7 +295,17 @@ main(int argc, char **argv)
          nir_shader *s = nir_precompiled_build_variant(
             libfunc, v, &agx_nir_options, &opt, load_kernel_input);
 
-         agx_link_libagx(s, nir);
+         nir_link_shader_functions(s, nir);
+         NIR_PASS(_, s, nir_inline_functions);
+         nir_remove_non_entrypoints(s);
+         NIR_PASS(_, s, nir_opt_deref);
+         NIR_PASS(_, s, nir_lower_vars_to_ssa);
+         NIR_PASS(_, s, nir_remove_dead_derefs);
+         NIR_PASS(_, s, nir_remove_dead_variables,
+                  nir_var_function_temp | nir_var_shader_temp, NULL);
+         NIR_PASS(_, s, nir_lower_vars_to_explicit_types,
+                  nir_var_shader_temp | nir_var_function_temp,
+                  glsl_get_cl_type_size_align);
 
          NIR_PASS(_, s, nir_lower_vars_to_explicit_types, nir_var_mem_shared,
                   glsl_get_cl_type_size_align);
@@ -301,7 +320,14 @@ main(int argc, char **argv)
             NIR_PASS(progress, s, nir_opt_loop);
          } while (progress);
 
-         agx_preprocess_nir(s, NULL);
+         agx_preprocess_nir(s);
+
+         NIR_PASS(_, s, nir_opt_deref);
+         NIR_PASS(_, s, nir_lower_vars_to_ssa);
+         NIR_PASS(_, s, nir_lower_explicit_io,
+                  nir_var_shader_temp | nir_var_function_temp |
+                     nir_var_mem_shared | nir_var_mem_global,
+                  nir_address_format_62bit_generic);
 
          bool has_atomic = false;
          nir_shader_intrinsics_pass(s, gather_atomic_info, nir_metadata_all,
@@ -319,7 +345,6 @@ main(int argc, char **argv)
             struct agx_shader_part compiled;
             bool is_helper = !strcmp(libfunc->name, "libagx_helper");
             struct agx_shader_key key = {
-               .libagx = nir,
                .promote_constants = !is_helper,
                .reserved_preamble = layout.size_B / 2,
                .is_helper = is_helper,
@@ -331,7 +356,7 @@ main(int argc, char **argv)
             }
 
             nir_shader *clone = nir_shader_clone(NULL, s);
-            agx_compile_shader_nir(clone, &key, NULL, &compiled);
+            agx_compile_shader_nir(clone, &key, &compiled);
             print_shader(fp_c, libfunc->name, *target, v, &compiled);
             free(compiled.binary);
             ralloc_free(clone);
@@ -357,11 +382,6 @@ main(int argc, char **argv)
       nir_precomp_print_binary_map(fp_c, nir, "libagx", *target, remap_variant);
    }
 
-   /* Remove the NIR functions we compiled to binaries to save memory */
-   nir_remove_entrypoints(nir);
-
-   nir_precomp_print_nir(fp_c, fp_h, nir, "libagx", "nir");
-
    glsl_type_singleton_decref();
    fclose(fp_c);
    fclose(fp_h);
