@@ -50,6 +50,9 @@
 
 #include <stdlib.h>
 
+#define XXH_INLINE_ALL
+#include "util/xxhash.h"
+
 struct intrinsic_info {
    nir_variable_mode mode; /* 0 if the mode is obtained from the deref. */
    nir_intrinsic_op op;
@@ -112,6 +115,8 @@ get_info(nir_intrinsic_op op)
       INFO(nir_var_mem_shared, shared_append_amd, true, -1, -1, -1, -1, 1)
       INFO(nir_var_mem_shared, shared_consume_amd, true, -1, -1, -1, -1, 1)
       LOAD(nir_var_mem_global, smem_amd, 0, 1, -1, 1)
+      LOAD(0, buffer_amd, 0, 1, -1, 1)
+      STORE(0, buffer_amd, 1, 2, -1, 0, 1)
    default:
       break;
 #undef ATOMIC
@@ -345,6 +350,12 @@ type_scalar_size_bytes(const struct glsl_type *type)
    return glsl_type_is_boolean(type) ? 4u : glsl_get_bit_size(type) / 8u;
 }
 
+static bool
+cmp_scalar(nir_scalar a, nir_scalar b)
+{
+   return a.def == b.def ? a.comp > b.comp : a.def->index > b.def->index;
+}
+
 static unsigned
 add_to_entry_key(nir_scalar *offset_defs, uint64_t *offset_defs_mul,
                  unsigned offset_def_count, nir_scalar def, uint64_t mul)
@@ -352,7 +363,7 @@ add_to_entry_key(nir_scalar *offset_defs, uint64_t *offset_defs_mul,
    mul = util_mask_sign_extend(mul, def.def->bit_size);
 
    for (unsigned i = 0; i <= offset_def_count; i++) {
-      if (i == offset_def_count || def.def->index > offset_defs[i].def->index) {
+      if (i == offset_def_count || cmp_scalar(def, offset_defs[i])) {
          /* insert before i */
          memmove(offset_defs + i + 1, offset_defs + i,
                  (offset_def_count - i) * sizeof(nir_scalar));
@@ -513,7 +524,9 @@ create_entry_key_from_offset(void *mem_ctx, nir_def *base, uint64_t base_mul, ui
 static nir_variable_mode
 get_variable_mode(struct entry *entry)
 {
-   if (entry->info->mode)
+   if (nir_intrinsic_has_memory_modes(entry->intrin))
+      return nir_intrinsic_memory_modes(entry->intrin);
+   else if (entry->info->mode)
       return entry->info->mode;
    assert(entry->deref && util_bitcount(entry->deref->modes) == 1);
    return entry->deref->modes;
@@ -543,6 +556,16 @@ aliasing_modes(nir_variable_mode modes)
 static void
 calc_alignment(struct entry *entry)
 {
+   if (entry->intrin->intrinsic == nir_intrinsic_load_buffer_amd ||
+       entry->intrin->intrinsic == nir_intrinsic_store_buffer_amd) {
+      /* The alignment is the result of the descriptor and several offset/index sources, so just use
+       * the original alignment.
+       */
+      entry->align_mul = nir_intrinsic_align_mul(entry->intrin);
+      entry->align_offset = nir_intrinsic_align_offset(entry->intrin);
+      return;
+   }
+
    uint32_t align_mul = 31;
    for (unsigned i = 0; i < entry->key->offset_def_count; i++) {
       if (entry->key->offset_defs_mul[i])
@@ -570,8 +593,7 @@ create_entry(void *mem_ctx,
    entry->info = info;
    entry->is_store = entry->info->value_src >= 0;
    entry->num_components =
-      entry->is_store ? intrin->num_components :
-                        nir_def_last_component_read(&intrin->def) + 1;
+      entry->is_store ? intrin->num_components : nir_def_last_component_read(&intrin->def) + 1;
 
    if (entry->info->deref_src >= 0) {
       entry->deref = nir_src_as_deref(intrin->src[entry->info->deref_src]);
@@ -770,11 +792,11 @@ vectorize_loads(nir_builder *b, struct vectorize_ctx *ctx,
    nir_def *high_undef = nir_undef(b, old_high_num_components, old_high_bit_size);
 
    nir_def *low_def = nir_extract_bits(
-      b, (nir_def*[]){data, low_undef}, 2, 0, old_low_num_components,
+      b, (nir_def *[]){ data, low_undef }, 2, 0, old_low_num_components,
       old_low_bit_size);
 
    nir_def *high_def = nir_extract_bits(
-      b, (nir_def*[]){data, high_undef}, 2, high_start,
+      b, (nir_def *[]){ data, high_undef }, 2, high_start,
       old_high_num_components, old_high_bit_size);
 
    /* convert booleans */
@@ -1031,6 +1053,24 @@ may_alias(nir_shader *shader, struct entry *a, struct entry *b)
    if (a->key->var != b->key->var || a->key->resource != b->key->resource)
       return true;
 
+   bool is_a_buffer_amd = a->intrin->intrinsic == nir_intrinsic_load_buffer_amd ||
+                          a->intrin->intrinsic == nir_intrinsic_store_buffer_amd;
+   bool is_b_buffer_amd = b->intrin->intrinsic == nir_intrinsic_load_buffer_amd ||
+                          b->intrin->intrinsic == nir_intrinsic_store_buffer_amd;
+   if (is_a_buffer_amd || is_b_buffer_amd) {
+      if (is_a_buffer_amd != is_b_buffer_amd)
+         return true;
+      if ((a->access | b->access) & ACCESS_USES_FORMAT_AMD)
+         return true;
+      bool a_store = a->intrin->intrinsic == nir_intrinsic_store_buffer_amd;
+      bool b_store = b->intrin->intrinsic == nir_intrinsic_store_buffer_amd;
+      /* Check the scalar byte offset and index offset. */
+      if (!nir_srcs_equal(a->intrin->src[2 + a_store], b->intrin->src[2 + b_store]))
+         return true;
+      if (!nir_srcs_equal(a->intrin->src[3 + a_store], b->intrin->src[3 + b_store]))
+         return true;
+   }
+
    /* use adjacency information */
    /* TODO: we can look closer at the entry keys */
    int64_t diff = compare_entries(a, b);
@@ -1179,6 +1219,20 @@ can_vectorize(struct vectorize_ctx *ctx, struct entry *first, struct entry *seco
        (first->access & ACCESS_VOLATILE) || first->info->is_atomic)
       return false;
 
+   if (first->intrin->intrinsic == nir_intrinsic_load_buffer_amd ||
+       first->intrin->intrinsic == nir_intrinsic_store_buffer_amd) {
+      if (first->access & ACCESS_USES_FORMAT_AMD)
+         return false;
+      if (nir_intrinsic_memory_modes(first->intrin) != nir_intrinsic_memory_modes(second->intrin))
+         return false;
+      bool store = first->intrin->intrinsic == nir_intrinsic_store_buffer_amd;
+      /* Check the scalar byte offset and index offset. */
+      if (!nir_srcs_equal(first->intrin->src[2 + store], second->intrin->src[2 + store]))
+         return false;
+      if (!nir_srcs_equal(first->intrin->src[3 + store], second->intrin->src[3 + store]))
+         return false;
+   }
+
    return true;
 }
 
@@ -1349,8 +1403,10 @@ vectorize_sorted_entries(struct vectorize_ctx *ctx, nir_function_impl *impl,
           */
          unsigned max_hole =
             first->is_store ||
-            (ctx->options->has_shared2_amd &&
-             get_variable_mode(first) == nir_var_mem_shared) ? 0 : 28;
+                  (ctx->options->has_shared2_amd &&
+                   get_variable_mode(first) == nir_var_mem_shared)
+               ? 0
+               : 28;
          unsigned low_size = get_bit_size(low) / 8u * low->num_components;
          bool separate = diff > max_hole + low_size;
 
@@ -1507,7 +1563,9 @@ process_block(nir_function_impl *impl, struct vectorize_ctx *ctx, nir_block *blo
          continue;
 
       nir_variable_mode mode = info->mode;
-      if (!mode)
+      if (nir_intrinsic_has_memory_modes(intrin))
+         mode = nir_intrinsic_memory_modes(intrin);
+      else if (!mode)
          mode = nir_src_as_deref(intrin->src[info->deref_src])->modes;
       if (!(mode & aliasing_modes(ctx->options->modes)))
          continue;
@@ -1572,9 +1630,8 @@ nir_opt_load_store_vectorize(nir_shader *shader, const nir_load_store_vectorize_
       nir_foreach_block(block, impl)
          progress |= process_block(impl, ctx, block);
 
-      nir_metadata_preserve(impl,
-                            nir_metadata_control_flow |
-                            nir_metadata_live_defs);
+      nir_progress(true, impl,
+                   nir_metadata_control_flow | nir_metadata_live_defs);
    }
 
    ralloc_free(ctx);
@@ -1606,6 +1663,7 @@ nir_opt_load_store_update_alignments(nir_shader *shader)
    return nir_shader_intrinsics_pass(shader,
                                      opt_load_store_update_alignments_callback,
                                      nir_metadata_control_flow |
-                                     nir_metadata_live_defs |
-                                     nir_metadata_instr_index, NULL);
+                                        nir_metadata_live_defs |
+                                        nir_metadata_instr_index,
+                                     NULL);
 }
