@@ -125,6 +125,7 @@ can_sink_instr(nir_instr *instr, nir_move_options options, bool *can_mov_out_of_
          return options & nir_move_copies;
       case nir_intrinsic_load_constant_agx:
       case nir_intrinsic_load_local_pixel_agx:
+      case nir_intrinsic_load_shader_output_pan:
          return true;
       default:
          return false;
@@ -261,8 +262,7 @@ nir_opt_sink(nir_shader *shader, nir_move_options options)
          }
       }
 
-      nir_metadata_preserve(impl,
-                            nir_metadata_control_flow);
+      nir_progress(true, impl, nir_metadata_control_flow);
    }
 
    return progress;
