@@ -33,6 +33,7 @@
 #include "util/half_float.h"
 #include "util/memstream.h"
 #include "util/mesa-blake3.h"
+#include "util/ralloc.h"
 #include "vulkan/vulkan_core.h"
 #include "nir.h"
 #include "nir_builder.h"
@@ -53,6 +54,9 @@ typedef struct {
    /** map from nir_variable -> printable name */
    struct hash_table *ht;
 
+   /** sorted list of block's predecessors  */
+   nir_block **preds;
+
    /** set of names used so far for nir_variables */
    struct set *syms;
 
@@ -80,7 +84,12 @@ typedef struct {
     */
    unsigned padding_for_no_dest;
 
-   nir_debug_info_instr **debug_info;
+   /* Whether divergence metadata is valid. */
+   bool divergence_valid;
+
+   bool gather_debug_info;
+
+   nir_instr_debug_info last_debug_info;
 } print_state;
 
 static void
@@ -110,7 +119,7 @@ static const char *sizes[] = { "x??", "   ", "x2 ", "x3 ", "x4 ",
 static const char *
 divergence_status(print_state *state, bool divergent)
 {
-   if (state->shader->info.divergence_analysis_run)
+   if (state->divergence_valid)
       return divergent ? "div " : "con ";
 
    return "";
@@ -135,12 +144,18 @@ print_def(nir_def *def, print_state *state)
            divergence_status(state, def->divergent),
            def->bit_size, sizes[def->num_components],
            padding, "", state->def_prefix, def->index);
+
+   if (state->shader->has_debug_info) {
+      nir_instr_debug_info *debug_info = nir_instr_get_debug_info(def->parent_instr);
+      if (debug_info->variable_name)
+         fprintf(fp, ".%s", debug_info->variable_name);
+   }
 }
 
 static unsigned
 calculate_padding_for_no_dest(print_state *state)
 {
-   const unsigned div = state->shader->info.divergence_analysis_run ? 4 : 0;
+   const unsigned div = state->divergence_valid ? 4 : 0;
    const unsigned ssa_size = 5;
    const unsigned percent = 1;
    const unsigned ssa_index = count_digits(state->max_dest_index);
@@ -202,19 +217,12 @@ print_hex_terse_const_value(const nir_const_value *value, unsigned bit_size, FIL
 static void
 print_float_const_value(const nir_const_value *value, unsigned bit_size, FILE *fp)
 {
-   switch (bit_size) {
-   case 64:
-      fprintf(fp, "%f", value->f64);
-      break;
-   case 32:
-      fprintf(fp, "%f", value->f32);
-      break;
-   case 16:
-      fprintf(fp, "%f", _mesa_half_to_float(value->u16));
-      break;
-   default:
-      unreachable("unhandled bit size");
-   }
+   double dval = nir_const_value_as_float(*value, bit_size);
+
+   if (fabs(dval) >= 1000000.0)
+      fprintf(fp, "%e", dval);
+   else
+      fprintf(fp, "%f", dval);
 }
 
 static void
@@ -398,6 +406,12 @@ print_src(const nir_src *src, print_state *state, nir_alu_type src_type)
    fprintf(fp, "%s%u", state->def_prefix, src->ssa->index);
    nir_instr *instr = src->ssa->parent_instr;
 
+   if (state->shader->has_debug_info) {
+      nir_instr_debug_info *debug_info = nir_instr_get_debug_info(instr);
+      if (debug_info->variable_name)
+         fprintf(fp, ".%s", debug_info->variable_name);
+   }
+
    if (instr->type == nir_instr_type_load_const && !NIR_DEBUG(PRINT_NO_INLINE_CONSTS)) {
       nir_load_const_instr *load_const = nir_instr_as_load_const(instr);
       fprintf(fp, " ");
@@ -795,15 +809,22 @@ print_access(enum gl_access_qualifier access, print_state *state, const char *se
       const char *name;
    } modes[] = {
       { ACCESS_COHERENT, "coherent" },
-      { ACCESS_VOLATILE, "volatile" },
       { ACCESS_RESTRICT, "restrict" },
+      { ACCESS_VOLATILE, "volatile" },
       { ACCESS_NON_WRITEABLE, "readonly" },
       { ACCESS_NON_READABLE, "writeonly" },
+      { ACCESS_NON_UNIFORM, "non-uniform" },
       { ACCESS_CAN_REORDER, "reorderable" },
-      { ACCESS_CAN_SPECULATE, "speculatable" },
       { ACCESS_NON_TEMPORAL, "non-temporal" },
       { ACCESS_INCLUDE_HELPERS, "include-helpers" },
+      { ACCESS_IS_SWIZZLED_AMD, "is-swizzled-amd" },
+      { ACCESS_USES_FORMAT_AMD, "uses-format-amd" },
+      { ACCESS_FMASK_LOWERED_AMD, "fmask-lowered-amd" },
+      { ACCESS_CAN_SPECULATE, "speculatable" },
       { ACCESS_CP_GE_COHERENT_AMD, "cp-ge-coherent-amd" },
+      { ACCESS_IN_BOUNDS, "in-bounds" },
+      { ACCESS_KEEP_SCALAR, "keep-scalar" },
+      { ACCESS_SMEM_AMD, "smem-amd" },
    };
 
    bool first = true;
@@ -830,8 +851,10 @@ print_var_decl(nir_variable *var, print_state *state)
    const char *const per_view = (var->data.per_view) ? "per_view " : "";
    const char *const per_primitive = (var->data.per_primitive) ? "per_primitive " : "";
    const char *const ray_query = (var->data.ray_query) ? "ray_query " : "";
-   fprintf(fp, "%s%s%s%s%s%s%s%s%s %s ",
-           bindless, cent, samp, patch, inv, per_view, per_primitive, ray_query,
+   const char *const fb_fetch = var->data.fb_fetch_output ? "fb_fetch_output " : "";
+   fprintf(fp, "%s%s%s%s%s%s%s%s%s%s %s ",
+           bindless, cent, samp, patch, inv, per_view, per_primitive,
+           ray_query, fb_fetch,
            get_variable_mode_str(var->data.mode, false),
            glsl_interp_mode_name(var->data.interpolation));
 
@@ -893,9 +916,9 @@ print_var_decl(nir_variable *var, print_state *state)
          fprintf(fp, " (%s%s)", loc, components);
       } else {
          fprintf(fp, " (%s%s, %u, %u)%s", loc,
-               components,
-               var->data.driver_location, var->data.binding,
-               var->data.compact ? " compact" : "");
+                 components,
+                 var->data.driver_location, var->data.binding,
+                 var->data.compact ? " compact" : "");
       }
    }
 
@@ -1355,13 +1378,17 @@ print_intrinsic_instr(nir_intrinsic_instr *instr, print_state *state)
          case nir_intrinsic_load_per_vertex_input:
          case nir_intrinsic_load_input_vertex:
          case nir_intrinsic_load_coefficients_agx:
+         case nir_intrinsic_load_attribute_pan:
             mode = nir_var_shader_in;
             break;
 
          case nir_intrinsic_load_output:
+         case nir_intrinsic_load_per_vertex_output:
+         case nir_intrinsic_load_per_primitive_output:
          case nir_intrinsic_store_output:
          case nir_intrinsic_store_per_primitive_output:
          case nir_intrinsic_store_per_vertex_output:
+         case nir_intrinsic_store_per_view_output:
             mode = nir_var_shader_out;
             break;
 
@@ -1386,6 +1413,9 @@ print_intrinsic_instr(nir_intrinsic_instr *instr, print_state *state)
          if (io.fb_fetch_output)
             fprintf(fp, " fbfetch");
 
+         if (io.fb_fetch_output_coherent)
+            fprintf(fp, " coherent");
+
          if (io.per_view)
             fprintf(fp, " perview");
 
@@ -1411,7 +1441,8 @@ print_intrinsic_instr(nir_intrinsic_instr *instr, print_state *state)
              state->shader->info.stage == MESA_SHADER_GEOMETRY &&
              (instr->intrinsic == nir_intrinsic_store_output ||
               instr->intrinsic == nir_intrinsic_store_per_primitive_output ||
-              instr->intrinsic == nir_intrinsic_store_per_vertex_output)) {
+              instr->intrinsic == nir_intrinsic_store_per_vertex_output ||
+              instr->intrinsic == nir_intrinsic_store_per_view_output)) {
             unsigned gs_streams = io.gs_streams;
             fprintf(fp, " gs_streams(");
             for (unsigned i = 0; i < 4; i++) {
@@ -1639,6 +1670,7 @@ print_intrinsic_instr(nir_intrinsic_instr *instr, print_state *state)
    case nir_intrinsic_load_output:
    case nir_intrinsic_store_output:
    case nir_intrinsic_store_per_vertex_output:
+   case nir_intrinsic_store_per_view_output:
       var_mode = nir_var_shader_out;
       break;
    default:
@@ -1727,6 +1759,9 @@ print_tex_instr(nir_tex_instr *instr, print_state *state)
    case nir_texop_tex_prefetch:
       fprintf(fp, "tex (pre-dispatchable) ");
       break;
+   case nir_texop_lod_bias:
+      fprintf(fp, "lod_bias ");
+      break;
    case nir_texop_fragment_fetch_amd:
       fprintf(fp, "fragment_fetch_amd ");
       break;
@@ -1739,8 +1774,8 @@ print_tex_instr(nir_tex_instr *instr, print_state *state)
    case nir_texop_sampler_descriptor_amd:
       fprintf(fp, "sampler_descriptor_amd ");
       break;
-   case nir_texop_lod_bias_agx:
-      fprintf(fp, "lod_bias_agx ");
+   case nir_texop_image_min_lod_agx:
+      fprintf(fp, "image_min_lod_agx ");
       break;
    case nir_texop_has_custom_border_color_agx:
       fprintf(fp, "has_custom_border_color_agx ");
@@ -1802,6 +1837,9 @@ print_tex_instr(nir_tex_instr *instr, print_state *state)
       case nir_tex_src_ms_mcs_intel:
          fprintf(fp, "(ms_mcs_intel)");
          break;
+      case nir_tex_src_lod_bias_min_agx:
+         fprintf(fp, "(lod_bias_min_agx)");
+         break;
       case nir_tex_src_ddx:
          fprintf(fp, "(ddx)");
          break;
@@ -1889,12 +1927,22 @@ print_call_instr(nir_call_instr *instr, print_state *state)
 
    print_no_dest_padding(state);
 
+   bool indirect = instr->indirect_callee.ssa;
+
    fprintf(fp, "call %s ", instr->callee->name);
+   if (indirect) {
+      fprintf(fp, "(indirect ");
+      print_src(&instr->indirect_callee, state, nir_type_invalid);
+      fprintf(fp, ") ");
+   }
 
    for (unsigned i = 0; i < instr->num_params; i++) {
       if (i != 0)
          fprintf(fp, ", ");
 
+      if (instr->callee->params[i].name)
+         fprintf(fp, "%s ", instr->callee->params[i].name);
+
       print_src(&instr->params[i], state, nir_type_invalid);
    }
 }
@@ -1952,13 +2000,20 @@ print_phi_instr(nir_phi_instr *instr, print_state *state)
    FILE *fp = state->fp;
    print_def(&instr->def, state);
    fprintf(fp, " = phi ");
-   nir_foreach_phi_src(src, instr) {
-      if (&src->node != exec_list_get_head(&instr->srcs))
+   nir_block **preds =
+      state->preds ? state->preds : nir_block_get_predecessors_sorted(instr->instr.block, NULL);
+
+   for (unsigned i = 0; i < instr->instr.block->predecessors->entries; i++) {
+      nir_phi_src *src = nir_phi_get_src_from_block(instr, preds[i]);
+      if (i != 0)
          fprintf(fp, ", ");
 
-      fprintf(fp, "b%u: ", src->pred->index);
+      fprintf(fp, "b%u: ", preds[i]->index);
       print_src(&src->src, state, nir_type_invalid);
    }
+
+   if (!state->preds)
+      ralloc_free(preds);
 }
 
 static void
@@ -1983,32 +2038,37 @@ print_parallel_copy_instr(nir_parallel_copy_instr *instr, print_state *state)
    }
 }
 
-static void
-print_debug_info_instr(nir_debug_info_instr *instr, print_state *state)
-{
-   FILE *fp = state->fp;
-
-   switch (instr->type) {
-   case nir_debug_info_src_loc:
-      fprintf(fp, "// 0x%x", instr->src_loc.spirv_offset);
-      if (instr->src_loc.line)
-         fprintf(fp, " %s:%u:%u", nir_src_as_string(instr->src_loc.filename), instr->src_loc.line, instr->src_loc.column);
-      return;
-   case nir_debug_info_string:
-      return; /* Strings are printed for their uses. */
-   }
-
-   unreachable("Unimplemented nir_debug_info_type");
-}
-
 static void
 print_instr(const nir_instr *instr, print_state *state, unsigned tabs)
 {
    FILE *fp = state->fp;
 
-   if (state->debug_info) {
-      nir_debug_info_instr *di = state->debug_info[instr->index];
-      di->src_loc.column = (uint32_t)ftell(fp);
+   if (state->gather_debug_info) {
+      nir_instr_debug_info *debug_info = nir_instr_get_debug_info((nir_instr *)instr);
+      debug_info->nir_line = (uint32_t)ftell(fp);
+   }
+
+   if (state->shader->has_debug_info && !state->gather_debug_info) {
+      nir_instr_debug_info *debug_info = nir_instr_get_debug_info((nir_instr *)instr);
+
+      bool changed = state->last_debug_info.spirv_offset != debug_info->spirv_offset;
+      changed |= state->last_debug_info.nir_line != debug_info->nir_line;
+
+      if (changed && debug_info->spirv_offset) {
+         print_indentation(tabs, fp);
+
+         fprintf(fp, "// 0x%x", debug_info->spirv_offset);
+
+         if (debug_info->filename)
+            fprintf(fp, " %s:%u:%u", debug_info->filename, debug_info->line, debug_info->column);
+
+         if (debug_info->nir_line)
+            fprintf(fp, " nir:%u", debug_info->nir_line);
+
+         fprintf(fp, "\n");
+      }
+
+      state->last_debug_info = *debug_info;
    }
 
    print_indentation(tabs, fp);
@@ -2054,10 +2114,6 @@ print_instr(const nir_instr *instr, print_state *state, unsigned tabs)
       print_parallel_copy_instr(nir_instr_as_parallel_copy(instr), state);
       break;
 
-   case nir_instr_type_debug_info:
-      print_debug_info_instr(nir_instr_as_debug_info(instr), state);
-      break;
-
    default:
       unreachable("Invalid instruction type");
       break;
@@ -2093,7 +2149,6 @@ block_has_instruction_with_dest(nir_block *block)
 
       case nir_instr_type_jump:
       case nir_instr_type_call:
-      case nir_instr_type_debug_info:
          /* Doesn't define a new value. */
          break;
       }
@@ -2109,13 +2164,9 @@ static void
 print_block_preds(nir_block *block, print_state *state)
 {
    FILE *fp = state->fp;
-   nir_block **preds = nir_block_get_predecessors_sorted(block, NULL);
    for (unsigned i = 0; i < block->predecessors->entries; i++) {
-      if (i != 0)
-         fprintf(fp, " ");
-      fprintf(fp, "b%u", preds[i]->index);
+      fprintf(fp, " b%u", state->preds[i]->index);
    }
-   ralloc_free(preds);
 }
 
 static void
@@ -2124,7 +2175,7 @@ print_block_succs(nir_block *block, print_state *state)
    FILE *fp = state->fp;
    for (unsigned i = 0; i < 2; i++) {
       if (block->successors[i]) {
-         fprintf(fp, "b%u ", block->successors[i]->index);
+         fprintf(fp, " b%u", block->successors[i]->index);
       }
    }
 }
@@ -2145,19 +2196,21 @@ print_block(nir_block *block, print_state *state, unsigned tabs)
            block->index);
 
    const bool empty_block = exec_list_is_empty(&block->instr_list);
+   state->preds = nir_block_get_predecessors_sorted(block, NULL);
    if (empty_block) {
-      fprintf(fp, "  // preds: ");
+      fprintf(fp, "  // preds:");
       print_block_preds(block, state);
-      fprintf(fp, ", succs: ");
+      fprintf(fp, ", succs:");
       print_block_succs(block, state);
       fprintf(fp, "\n");
+      ralloc_free(state->preds);
       return;
    }
 
    const unsigned block_length = 7 + count_digits(block->index) + 1;
    const unsigned pred_padding = block_length < state->padding_for_no_dest ? state->padding_for_no_dest - block_length : 0;
 
-   fprintf(fp, "%*s// preds: ", pred_padding, "");
+   fprintf(fp, "%*s// preds:", pred_padding, "");
    print_block_preds(block, state);
    fprintf(fp, "\n");
 
@@ -2168,9 +2221,10 @@ print_block(nir_block *block, print_state *state, unsigned tabs)
    }
 
    print_indentation(tabs, fp);
-   fprintf(fp, "%*s// succs: ", state->padding_for_no_dest, "");
+   fprintf(fp, "%*s// succs:", state->padding_for_no_dest, "");
    print_block_succs(block, state);
    fprintf(fp, "\n");
+   ralloc_free(state->preds);
 }
 
 static void
@@ -2253,15 +2307,18 @@ print_cf_node(nir_cf_node *node, print_state *state, unsigned int tabs)
 }
 
 static void
-print_function_impl(nir_function_impl *impl, print_state *state)
+print_function_impl(nir_function_impl *impl, print_state *state, bool print_name)
 {
    FILE *fp = state->fp;
 
    state->max_dest_index = impl->ssa_alloc;
+   state->divergence_valid = impl->valid_metadata & nir_metadata_divergence;
 
-   fprintf(fp, "\nimpl %s ", impl->function->name);
+   if (print_name) {
+      fprintf(fp, "\nimpl %s ", impl->function->name);
 
-   fprintf(fp, "{\n");
+      fprintf(fp, "{\n");
+   }
 
    if (impl->preamble) {
       print_indentation(1, fp);
@@ -2290,7 +2347,11 @@ print_function_impl(nir_function_impl *impl, print_state *state)
    }
 
    print_indentation(1, fp);
-   fprintf(fp, "block b%u:\n}\n\n", impl->end_block->index);
+   fprintf(fp, "block b%u:\n", impl->end_block->index);
+
+   if (print_name) {
+      fprintf(fp, "}\n\n");
+   }
 
    free(state->float_types);
    free(state->int_types);
@@ -2302,18 +2363,47 @@ print_function(nir_function *function, print_state *state)
 {
    FILE *fp = state->fp;
 
+   fprintf(fp, "decl_function %s (", function->name);
+
+   for (unsigned i = 0; i < function->num_params; ++i) {
+      if (i != 0) {
+         fprintf(fp, ", ");
+      }
+
+      nir_parameter param = function->params[i];
+
+      fprintf(fp, "%u", param.bit_size);
+      if (param.num_components != 1) {
+         fprintf(fp, "x%u", param.num_components);
+      }
+
+      if (param.name) {
+         fprintf(fp, " %s", param.name);
+      } else if (param.is_return) {
+         fprintf(fp, " return");
+      }
+   }
+
+   fprintf(fp, ")");
+
    /* clang-format off */
-   fprintf(fp, "decl_function %s (%d params)%s%s", function->name,
-           function->num_params,
-           function->dont_inline ? " (noinline)" :
-           function->should_inline ? " (inline)" : "",
-           function->is_exported ? " (exported)" : "");
+   fprintf(fp, "%s%s%s", function->dont_inline ? " (noinline)" :
+                       function->should_inline ? " (inline)" : "",
+                       function->is_exported ? " (exported)" : "",
+                       function->is_entrypoint ? " (entrypoint)" : "");
    /* clang-format on */
 
+   if (function->workgroup_size[0]) {
+      fprintf(fp, " (%ux%ux%u)",
+              function->workgroup_size[0],
+              function->workgroup_size[1],
+              function->workgroup_size[2]);
+   }
+
    fprintf(fp, "\n");
 
    if (function->impl != NULL) {
-      print_function_impl(function->impl, state);
+      print_function_impl(function->impl, state, true);
       return;
    }
 }
@@ -2464,9 +2554,11 @@ print_shader_info(const struct shader_info *info, FILE *fp)
 {
    fprintf(fp, "shader: %s\n", gl_shader_stage_name(info->stage));
 
-   fprintf(fp, "source_blake3: {");
-   _mesa_blake3_print(fp, info->source_blake3);
-   fprintf(fp, "}\n");
+   if (memcmp(info->source_blake3, &(blake3_hash){ 0 }, sizeof(info->source_blake3))) {
+      fprintf(fp, "source_blake3: {");
+      _mesa_blake3_print(fp, info->source_blake3);
+      fprintf(fp, "}\n");
+   }
 
    if (info->name)
       fprintf(fp, "name: %s\n", info->name);
@@ -2474,7 +2566,7 @@ print_shader_info(const struct shader_info *info, FILE *fp)
    if (info->label)
       fprintf(fp, "label: %s\n", info->label);
 
-   fprintf(fp, "internal: %s\n", info->internal ? "true" : "false");
+   print_nz_bool(fp, "internal", info->internal);
 
    if (gl_shader_stage_uses_workgroup(info->stage)) {
       fprintf(fp, "workgroup_size: %u, %u, %u%s\n",
@@ -2484,9 +2576,8 @@ print_shader_info(const struct shader_info *info, FILE *fp)
               info->workgroup_size_variable ? " (variable)" : "");
    }
 
-   fprintf(fp, "stage: %d\n"
-               "next_stage: %d\n",
-           info->stage, info->next_stage);
+   if (info->next_stage != MESA_SHADER_NONE)
+      fprintf(fp, "next_stage: %s\n", gl_shader_stage_name(info->next_stage));
 
    print_nz_unsigned(fp, "num_textures", info->num_textures);
    print_nz_unsigned(fp, "num_ubos", info->num_ubos);
@@ -2563,7 +2654,6 @@ print_shader_info(const struct shader_info *info, FILE *fp)
 
    print_nz_bool(fp, "uses_texture_gather", info->uses_texture_gather);
    print_nz_bool(fp, "uses_resource_info_query", info->uses_resource_info_query);
-   print_nz_bool(fp, "divergence_analysis_run", info->divergence_analysis_run);
 
    print_nz_x8(fp, "bit_sizes_float", info->bit_sizes_float);
    print_nz_x8(fp, "bit_sizes_int", info->bit_sizes_int);
@@ -2614,7 +2704,8 @@ print_shader_info(const struct shader_info *info, FILE *fp)
       print_nz_bool(fp, "color_is_dual_source", info->fs.color_is_dual_source);
 
       print_nz_bool(fp, "require_full_quads", info->fs.require_full_quads);
-      print_nz_bool(fp, "needs_quad_helper_invocations", info->fs.needs_quad_helper_invocations);
+      print_nz_bool(fp, "needs_coarse_quad_helper_invocations", info->fs.needs_coarse_quad_helper_invocations);
+      print_nz_bool(fp, "needs_full_quad_helper_invocations", info->fs.needs_full_quad_helper_invocations);
       print_nz_bool(fp, "uses_sample_qualifier", info->fs.uses_sample_qualifier);
       print_nz_bool(fp, "uses_sample_shading", info->fs.uses_sample_shading);
       print_nz_bool(fp, "early_fragment_tests", info->fs.early_fragment_tests);
@@ -2656,7 +2747,7 @@ print_shader_info(const struct shader_info *info, FILE *fp)
                  info->cs.workgroup_size_hint[1],
                  info->cs.workgroup_size_hint[2]);
       print_nz_unsigned(fp, "user_data_components_amd", info->cs.user_data_components_amd);
-      fprintf(fp, "ptr_size: %u\n", info->cs.ptr_size);
+      print_nz_unsigned(fp, "ptr_size", info->cs.ptr_size);
       break;
 
    case MESA_SHADER_MESH:
@@ -2672,22 +2763,32 @@ print_shader_info(const struct shader_info *info, FILE *fp)
    }
 }
 
+void
+nir_print_function_body(nir_function_impl *impl, FILE *fp)
+{
+   print_state state = { 0 };
+   init_print_state(&state, impl->function->shader, fp);
+   state.def_prefix = "%";
+   print_function_impl(impl, &state, false);
+   destroy_print_state(&state);
+}
+
 static void
 _nir_print_shader_annotated(nir_shader *shader, FILE *fp,
                             struct hash_table *annotations,
-                            nir_debug_info_instr **debug_info)
+                            bool gather_debug_info)
 {
    print_state state;
    init_print_state(&state, shader, fp);
-   state.def_prefix = debug_info ? "ssa_" : "%";
+   state.def_prefix = gather_debug_info ? "ssa_" : "%";
    state.annotations = annotations;
-   state.debug_info = debug_info;
+   state.gather_debug_info = gather_debug_info;
 
    print_shader_info(&shader->info, fp);
 
-   fprintf(fp, "inputs: %u\n", shader->num_inputs);
-   fprintf(fp, "outputs: %u\n", shader->num_outputs);
-   fprintf(fp, "uniforms: %u\n", shader->num_uniforms);
+   print_nz_unsigned(fp, "inputs", shader->num_inputs);
+   print_nz_unsigned(fp, "outputs", shader->num_outputs);
+   print_nz_unsigned(fp, "uniforms", shader->num_uniforms);
    if (shader->scratch_size)
       fprintf(fp, "scratch: %u\n", shader->scratch_size);
    if (shader->constant_data_size)
@@ -2699,7 +2800,7 @@ _nir_print_shader_annotated(nir_shader *shader, FILE *fp,
 
       if (mode == nir_var_shader_in || mode == nir_var_shader_out) {
          for (unsigned j = 0; j < 128; j++) {
-            nir_variable *vars[NIR_MAX_VEC_COMPONENTS] = {0};
+            nir_variable *vars[NIR_MAX_VEC_COMPONENTS] = { 0 };
             nir_foreach_variable_with_modes(var, shader, mode) {
                if (var->data.location == j)
                   vars[var->data.location_frac] = var;
@@ -2726,7 +2827,7 @@ void
 nir_print_shader_annotated(nir_shader *shader, FILE *fp,
                            struct hash_table *annotations)
 {
-   _nir_print_shader_annotated(shader, fp, annotations, NULL);
+   _nir_print_shader_annotated(shader, fp, annotations, false);
 }
 
 void
@@ -2738,14 +2839,14 @@ nir_print_shader(nir_shader *shader, FILE *fp)
 
 static char *
 _nir_shader_as_str_annotated(nir_shader *nir, struct hash_table *annotations, void *mem_ctx,
-                             nir_debug_info_instr **debug_info)
+                             bool gather_debug_info)
 {
    char *stream_data = NULL;
    size_t stream_size = 0;
    struct u_memstream mem;
    if (u_memstream_open(&mem, &stream_data, &stream_size)) {
       FILE *const stream = u_memstream_get(&mem);
-      _nir_print_shader_annotated(nir, stream, annotations, debug_info);
+      _nir_print_shader_annotated(nir, stream, annotations, gather_debug_info);
       u_memstream_close(&mem);
    }
 
@@ -2761,7 +2862,7 @@ _nir_shader_as_str_annotated(nir_shader *nir, struct hash_table *annotations, vo
 char *
 nir_shader_as_str_annotated(nir_shader *nir, struct hash_table *annotations, void *mem_ctx)
 {
-   return _nir_shader_as_str_annotated(nir, annotations, mem_ctx, NULL);
+   return _nir_shader_as_str_annotated(nir, annotations, mem_ctx, false);
 }
 
 char *
@@ -2780,6 +2881,7 @@ nir_print_instr(const nir_instr *instr, FILE *fp)
    if (instr->block) {
       nir_function_impl *impl = nir_cf_node_get_function(&instr->block->cf_node);
       state.shader = impl->function->shader;
+      state.divergence_valid = impl->valid_metadata & nir_metadata_divergence;
    }
 
    print_instr(instr, &state, 0);
@@ -2826,65 +2928,27 @@ nir_log_shader_annotated_tagged(enum mesa_log_level level, const char *tag,
 }
 
 char *
-nir_shader_gather_debug_info(nir_shader *shader, const char *filename)
+nir_shader_gather_debug_info(nir_shader *shader, const char *filename, uint32_t first_line)
 {
-   uint32_t instr_count = 0;
-   nir_foreach_function_impl(impl, shader) {
-      nir_foreach_block(block, impl) {
-         nir_foreach_instr(instr, block) {
-            instr->index = instr_count;
-            instr_count++;
-         }
-      }
-   }
+   char *str = _nir_shader_as_str_annotated(shader, NULL, NULL, shader->has_debug_info);
 
-   if (!instr_count)
-      return nir_shader_as_str(shader, NULL);
+   if (!shader->has_debug_info)
+      return str;
 
-   nir_debug_info_instr **debug_info = rzalloc_array(shader, nir_debug_info_instr *, instr_count);
-
-   instr_count = 0;
-   nir_foreach_function_impl(impl, shader) {
-      nir_builder b = nir_builder_at(nir_before_cf_list(&impl->body));
-      nir_def *filename_def = nir_build_string(&b, filename);
-
-      nir_foreach_block(block, impl) {
-         nir_foreach_instr_safe(instr, block) {
-            if (instr->type == nir_instr_type_debug_info)
-               continue;
-
-            nir_debug_info_instr *di = nir_debug_info_instr_create(shader, nir_debug_info_src_loc, 0);
-            di->src_loc.filename = nir_src_for_ssa(filename_def);
-            di->src_loc.source = nir_debug_info_nir;
-            debug_info[instr_count++] = di;
-         }
-      }
-   }
-
-   char *str = _nir_shader_as_str_annotated(shader, NULL, NULL, debug_info);
-
-   uint32_t line = 1;
+   uint32_t line = first_line;
    uint32_t character_index = 0;
 
-   for (uint32_t i = 0; i < instr_count; i++) {
-      nir_debug_info_instr *di = debug_info[i];
-
-      while (character_index < di->src_loc.column) {
-         if (str[character_index] == '\n')
-            line++;
-         character_index++;
-      }
-
-      di->src_loc.line = line;
-      di->src_loc.column = 0;
-   }
-
-   instr_count = 0;
    nir_foreach_function_impl(impl, shader) {
       nir_foreach_block(block, impl) {
          nir_foreach_instr_safe(instr, block) {
-            if (instr->type != nir_instr_type_debug_info)
-               nir_instr_insert_before(instr, &debug_info[instr_count++]->instr);
+            nir_instr_debug_info *di = nir_instr_get_debug_info(instr);
+            while (character_index < di->nir_line) {
+               if (str[character_index] == '\n')
+                  line++;
+               character_index++;
+            }
+
+            di->nir_line = line;
          }
       }
    }
