@@ -80,6 +80,8 @@ struct ir3_sched_ctx {
    struct ir3_instruction *scheduled; /* last scheduled instr */
    struct ir3_instruction *addr0;     /* current a0.x user, if any */
    struct ir3_instruction *addr1;     /* current a1.x user, if any */
+   unsigned addr0_uses;               /* number of unscheduled uses of addr0 */
+   unsigned addr1_uses;               /* number of unscheduled uses of addr1 */
 
    struct ir3_instruction *split; /* most-recently-split a0/a1 producer */
 
@@ -256,11 +258,31 @@ schedule(struct ir3_sched_ctx *ctx, struct ir3_instruction *instr)
    if (writes_addr0(instr)) {
       assert(ctx->addr0 == NULL);
       ctx->addr0 = instr;
+      ctx->addr0_uses = instr->uses->entries;
    }
 
    if (writes_addr1(instr)) {
       assert(ctx->addr1 == NULL);
       ctx->addr1 = instr;
+      ctx->addr1_uses = instr->uses->entries;
+   }
+
+   if (reads_addr0(instr)) {
+      assert(instr->address->def->instr == ctx->addr0);
+      assert(ctx->addr0_uses > 0);
+
+      if (--ctx->addr0_uses == 0) {
+         ctx->addr0 = NULL;
+      }
+   }
+
+   if (reads_addr1(instr)) {
+      assert(instr->address->def->instr == ctx->addr1);
+      assert(ctx->addr1_uses > 0);
+
+      if (--ctx->addr1_uses == 0) {
+         ctx->addr1 = NULL;
+      }
    }
 
    instr->flags |= IR3_INSTR_MARK;
@@ -914,8 +936,10 @@ split_addr(struct ir3_sched_ctx *ctx, struct ir3_instruction **addr,
             new_addr = split_instr(ctx, *addr);
             /* original addr is scheduled, but new one isn't: */
             new_addr->flags &= ~IR3_INSTR_MARK;
+            new_addr->uses = _mesa_pointer_set_create(ctx);
          }
          indirect->address->def = new_addr->dsts[0];
+         _mesa_set_add(new_addr->uses, indirect);
          /* don't need to remove old dag edge since old addr is
           * already scheduled:
           */
@@ -1348,6 +1372,29 @@ add_barrier_deps(struct ir3_block *block, struct ir3_instruction *instr)
    }
 }
 
+static bool
+add_const_deps(struct ir3_block *block, struct ir3_instruction *stc)
+{
+   bool progress = false;
+   unsigned const_start = stc->cat6.dst_offset;
+   unsigned const_end = const_start + stc->cat6.iim_val;
+
+   foreach_instr_from (instr, stc, &block->instr_list) {
+      foreach_src (src, instr) {
+         if (!(src->flags & IR3_REG_CONST)) {
+            continue;
+         }
+
+         if (src->num >= const_start && src->num < const_end) {
+            ir3_instr_add_dep(instr, stc);
+            progress = true;
+         }
+      }
+   }
+
+   return progress;
+}
+
 /* before scheduling a block, we need to add any necessary false-dependencies
  * to ensure that:
  *
@@ -1368,6 +1415,10 @@ ir3_sched_add_deps(struct ir3 *ir)
             add_barrier_deps(block, instr);
             progress = true;
          }
+
+         if (instr->opc == OPC_STC) {
+            progress |= add_const_deps(block, instr);
+         }
       }
    }
 
