@@ -56,6 +56,7 @@
 #include "dev/intel_device_info.h"
 #include "drm-uapi/dma-buf.h"
 #include "isl/isl.h"
+#include "util/os_file.h"
 #include "util/os_mman.h"
 #include "util/u_debug.h"
 #include "util/macros.h"
@@ -285,7 +286,7 @@ find_and_ref_external_bo(struct hash_table *ht, unsigned int key)
  */
 static struct bo_cache_bucket *
 bucket_for_size(struct iris_bufmgr *bufmgr, uint64_t size,
-                enum iris_heap heap, unsigned flags)
+                enum iris_heap heap, enum bo_alloc_flags flags)
 {
    if (flags & BO_ALLOC_PROTECTED)
       return NULL;
@@ -722,7 +723,7 @@ iris_slab_alloc(void *priv,
 {
    struct iris_bufmgr *bufmgr = priv;
    struct iris_slab *slab = calloc(1, sizeof(struct iris_slab));
-   uint32_t flags = BO_ALLOC_NO_SUBALLOC;
+   enum bo_alloc_flags flags = BO_ALLOC_NO_SUBALLOC;
    unsigned slab_size = 0;
    /* We only support slab allocation for IRIS_MEMZONE_OTHER */
    enum iris_memory_zone memzone = IRIS_MEMZONE_OTHER;
@@ -846,7 +847,7 @@ fail:
  * This determines the cacheability, coherency, and mmap mode settings.
  */
 static enum iris_heap
-flags_to_heap(struct iris_bufmgr *bufmgr, unsigned flags)
+flags_to_heap(struct iris_bufmgr *bufmgr, enum bo_alloc_flags flags)
 {
    const struct intel_device_info *devinfo = &bufmgr->devinfo;
 
@@ -894,7 +895,7 @@ flags_to_heap(struct iris_bufmgr *bufmgr, unsigned flags)
 
 static bool
 zero_bo(struct iris_bufmgr *bufmgr,
-        unsigned flags,
+        enum bo_alloc_flags flags,
         struct iris_bo *bo)
 {
    assert(flags & BO_ALLOC_ZEROED);
@@ -924,7 +925,7 @@ alloc_bo_from_slabs(struct iris_bufmgr *bufmgr,
                     const char *name,
                     uint64_t size,
                     uint32_t alignment,
-                    unsigned flags)
+                    enum bo_alloc_flags flags)
 {
    if (flags & BO_ALLOC_NO_SUBALLOC)
       return NULL;
@@ -1009,7 +1010,7 @@ alloc_bo_from_cache(struct iris_bufmgr *bufmgr,
                     uint32_t alignment,
                     enum iris_memory_zone memzone,
                     enum iris_mmap_mode mmap_mode,
-                    unsigned flags,
+                    enum bo_alloc_flags flags,
                     bool match_zone)
 {
    if (!bucket)
@@ -1106,7 +1107,7 @@ alloc_bo_from_cache(struct iris_bufmgr *bufmgr,
 }
 
 static struct iris_bo *
-alloc_fresh_bo(struct iris_bufmgr *bufmgr, uint64_t bo_size, unsigned flags)
+alloc_fresh_bo(struct iris_bufmgr *bufmgr, uint64_t bo_size, enum bo_alloc_flags flags)
 {
    struct iris_bo *bo = bo_calloc();
    if (!bo)
@@ -1212,7 +1213,7 @@ iris_bo_alloc(struct iris_bufmgr *bufmgr,
               uint64_t size,
               uint32_t alignment,
               enum iris_memory_zone memzone,
-              unsigned flags)
+              enum bo_alloc_flags flags)
 {
    struct iris_bo *bo;
    unsigned int page_size = getpagesize();
@@ -1271,7 +1272,7 @@ iris_bo_alloc(struct iris_bufmgr *bufmgr,
       if (bo->address == 0ull)
          goto err_free;
 
-      if (!bufmgr->kmd_backend->gem_vm_bind(bo))
+      if (!bufmgr->kmd_backend->gem_vm_bind(bo, flags))
          goto err_vm_alloc;
    }
 
@@ -1359,7 +1360,7 @@ iris_bo_create_userptr(struct iris_bufmgr *bufmgr, const char *name,
    bo->real.mmap_mode = heap_to_mmap_mode(bufmgr, bo->real.heap);
    bo->real.prime_fd = -1;
 
-   if (!bufmgr->kmd_backend->gem_vm_bind(bo))
+   if (!bufmgr->kmd_backend->gem_vm_bind(bo, 0))
       goto err_vma_free;
 
    return bo;
@@ -1406,7 +1407,8 @@ iris_bo_set_prime_fd(struct iris_bo *bo)
  */
 struct iris_bo *
 iris_bo_gem_create_from_name(struct iris_bufmgr *bufmgr,
-                             const char *name, unsigned int handle)
+                             const char *name, unsigned int handle,
+                             unsigned flags)
 {
    struct iris_bo *bo;
 
@@ -1469,7 +1471,7 @@ iris_bo_gem_create_from_name(struct iris_bufmgr *bufmgr,
    if (!iris_bo_set_prime_fd(bo))
       goto err_vm_alloc;
 
-   if (!bufmgr->kmd_backend->gem_vm_bind(bo))
+   if (!bufmgr->kmd_backend->gem_vm_bind(bo, flags))
       goto err_vm_alloc;
 
    _mesa_hash_table_insert(bufmgr->handle_table, &bo->gem_handle, bo);
@@ -1928,7 +1930,7 @@ iris_gem_set_tiling(struct iris_bo *bo, const struct isl_surf *surf)
 
 struct iris_bo *
 iris_bo_import_dmabuf(struct iris_bufmgr *bufmgr, int prime_fd,
-                      const uint64_t modifier)
+                      const uint64_t modifier, unsigned flags)
 {
    uint32_t handle;
    struct iris_bo *bo;
@@ -1977,7 +1979,7 @@ iris_bo_import_dmabuf(struct iris_bufmgr *bufmgr, int prime_fd,
    if (INTEL_DEBUG(DEBUG_CAPTURE_ALL))
       bo->real.capture = true;
    bo->gem_handle = handle;
-   bo->real.prime_fd = needs_prime_fd(bufmgr) ? dup(prime_fd) : -1;
+   bo->real.prime_fd = needs_prime_fd(bufmgr) ? os_dupfd_cloexec(prime_fd) : -1;
 
    uint64_t alignment = 1;
 
@@ -1992,7 +1994,7 @@ iris_bo_import_dmabuf(struct iris_bufmgr *bufmgr, int prime_fd,
    if (bo->address == 0ull)
       goto err_free;
 
-   if (!bufmgr->kmd_backend->gem_vm_bind(bo))
+   if (!bufmgr->kmd_backend->gem_vm_bind(bo, flags))
       goto err_vm_alloc;
 
    _mesa_hash_table_insert(bufmgr->handle_table, &bo->gem_handle, bo);
@@ -2240,7 +2242,7 @@ intel_aux_map_buffer_alloc(void *driver_ctx, uint32_t size)
    if (bo->address == 0ull)
       goto err_free;
 
-   if (!bufmgr->kmd_backend->gem_vm_bind(bo))
+   if (!bufmgr->kmd_backend->gem_vm_bind(bo, 0))
       goto err_vm_alloc;
 
    simple_mtx_unlock(&bufmgr->lock);
