@@ -302,8 +302,11 @@ static VkResult lvp_get_image_format_properties(struct lvp_physical_device *phys
                                              &format_props);
    if (info->tiling == VK_IMAGE_TILING_LINEAR) {
       format_feature_flags = format_props.linearTilingFeatures;
-   } else if (info->tiling == VK_IMAGE_TILING_OPTIMAL ||
-              info->tiling == VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT) {
+   } else if (info->tiling == VK_IMAGE_TILING_OPTIMAL) {
+      format_feature_flags = format_props.optimalTilingFeatures;
+   } else if (info->tiling == VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT) {
+      if (vk_format_get_plane_count (info->format) > 1)
+        goto unsupported;
       format_feature_flags = format_props.optimalTilingFeatures;
    } else {
       unreachable("bad VkImageTiling");
@@ -312,8 +315,8 @@ static VkResult lvp_get_image_format_properties(struct lvp_physical_device *phys
    if (format_feature_flags == 0)
       goto unsupported;
 
-   uint32_t max_2d_ext = physical_device->pscreen->get_param(physical_device->pscreen, PIPE_CAP_MAX_TEXTURE_2D_SIZE);
-   uint32_t max_layers = physical_device->pscreen->get_param(physical_device->pscreen, PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS);
+   uint32_t max_2d_ext = physical_device->pscreen->caps.max_texture_2d_size;
+   uint32_t max_layers = physical_device->pscreen->caps.max_texture_array_layers;
    switch (info->type) {
    default:
       unreachable("bad vkimage type\n");
@@ -342,7 +345,7 @@ static VkResult lvp_get_image_format_properties(struct lvp_physical_device *phys
    case VK_IMAGE_TYPE_3D:
       maxExtent.width = max_2d_ext;
       maxExtent.height = max_2d_ext;
-      maxExtent.depth = (1 << physical_device->pscreen->get_param(physical_device->pscreen, PIPE_CAP_MAX_TEXTURE_3D_LEVELS));
+      maxExtent.depth = (1 << physical_device->pscreen->caps.max_texture_3d_levels);
       maxMipLevels = util_logbase2(max_2d_ext) + 1;
       maxArraySize = 1;
       break;
@@ -472,7 +475,7 @@ VKAPI_ATTR VkResult VKAPI_CALL lvp_GetPhysicalDeviceImageFormatProperties2(
       switch (external_info->handleType) {
 #ifdef HAVE_LIBDRM
       case VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT: {
-         int params = physical_device->pscreen->get_param(physical_device->pscreen, PIPE_CAP_DMABUF);
+         int params = physical_device->pscreen->caps.dmabuf;
          flags = VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
          if (params & DRM_PRIME_CAP_EXPORT)
             flags |= VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT;
@@ -633,7 +636,7 @@ VKAPI_ATTR void VKAPI_CALL lvp_GetPhysicalDeviceExternalBufferProperties(
 #ifdef HAVE_LIBDRM
       case VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT: {
          LVP_FROM_HANDLE(lvp_physical_device, physical_device, physicalDevice);
-         int params = physical_device->pscreen->get_param(physical_device->pscreen, PIPE_CAP_DMABUF);
+         int params = physical_device->pscreen->caps.dmabuf;
          flags = VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
          if (params & DRM_PRIME_CAP_EXPORT)
             flags |= VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT;
