@@ -157,6 +157,7 @@ impl PipeResource {
         format: pipe_format,
         read_write: bool,
         size: u32,
+        offset_bytes: u32,
     ) -> PipeImageView {
         debug_assert!(self.is_buffer());
 
@@ -173,7 +174,7 @@ impl PipeResource {
             shader_access: shader_access,
             u: pipe_image_view__bindgen_ty_1 {
                 buf: pipe_image_view__bindgen_ty_1__bindgen_ty_2 {
-                    offset: 0,
+                    offset: offset_bytes,
                     size: size,
                 },
             },
@@ -185,6 +186,7 @@ impl PipeResource {
         format: pipe_format,
         read_write: bool,
         app_img_info: &AppImgInfo,
+        offset_pixels: u32,
     ) -> PipeImageView {
         debug_assert!(self.is_buffer());
 
@@ -201,7 +203,7 @@ impl PipeResource {
             shader_access: shader_access,
             u: pipe_image_view__bindgen_ty_1 {
                 tex2d_from_buf: pipe_tex2d_from_buf {
-                    offset: 0,
+                    offset: offset_pixels,
                     row_stride: app_img_info.row_stride as u16,
                     width: app_img_info.width as u16,
                     height: app_img_info.height as u16,
@@ -225,6 +227,7 @@ impl PipeResource {
         &self,
         format: pipe_format,
         size: u32,
+        offset_bytes: u32,
     ) -> pipe_sampler_view {
         debug_assert!(self.is_buffer());
 
@@ -236,7 +239,7 @@ impl PipeResource {
         // write the entire union field because u_sampler_view_default_template might have left it
         // in an undefined state.
         res.u.buf = pipe_sampler_view__bindgen_ty_1__bindgen_ty_2 {
-            offset: 0,
+            offset: offset_bytes,
             size: size,
         };
 
@@ -247,6 +250,7 @@ impl PipeResource {
         &self,
         format: pipe_format,
         app_img_info: &AppImgInfo,
+        offset_pixels: u32,
     ) -> pipe_sampler_view {
         debug_assert!(self.is_buffer());
 
@@ -258,7 +262,7 @@ impl PipeResource {
         // write the entire union field because u_sampler_view_default_template might have left it
         // in an undefined state.
         res.u.tex2d_from_buf = pipe_tex2d_from_buf {
-            offset: 0,
+            offset: offset_pixels,
             row_stride: app_img_info.row_stride as u16,
             width: app_img_info.width as u16,
             height: app_img_info.height as u16,
