@@ -247,6 +247,20 @@ validate_color_buffer(struct gl_context *ctx, struct gl_framebuffer *readFb,
 
    }
 
+   /**
+    * OVR_multiview
+
+      INVALID_FRAMEBUFFER_OPERATION is generated by commands that read from the
+      framebuffer such as BlitFramebuffer, ReadPixels, CopyTexImage*, and
+      CopyTexSubImage*, if the number of views in the current read framebuffer
+      is greater than 1.
+      */
+   if (colorReadRb->rtt_numviews > 1) {
+      _mesa_error(ctx, GL_INVALID_FRAMEBUFFER_OPERATION,
+                  "%s(NumViews > 1 on read framebuffer)", func);
+      return false;
+   }
+
    if (filter != GL_NEAREST) {
       /* From EXT_framebuffer_multisample_blit_scaled specification:
        * "Calling BlitFramebuffer will result in an INVALID_OPERATION error if
