@@ -24,11 +24,7 @@
 #ifndef FORMATQUERY_H
 #define FORMATQUERY_H
 
-#include "glheader.h"
-
-size_t
-_mesa_query_samples_for_format(struct gl_context *ctx, GLenum target,
-                               GLenum internalFormat, int samples[16]);
+#include "util/glheader.h"
 
 void
 _mesa_query_internal_format_default(struct gl_context *ctx, GLenum target,
