@@ -147,9 +147,13 @@ panvk_AllocateMemory(VkDevice _device,
 
    if (device->debug.decode_ctx) {
       if (instance->debug_flags & (PANVK_DEBUG_DUMP | PANVK_DEBUG_TRACE)) {
-         mem->debug.host_mapping =
-            pan_kmod_bo_mmap(mem->bo, 0, pan_kmod_bo_size(mem->bo),
-                             PROT_READ | PROT_WRITE, MAP_SHARED, NULL);
+         void *cpu = pan_kmod_bo_mmap(mem->bo, 0, pan_kmod_bo_size(mem->bo),
+                                      PROT_READ | PROT_WRITE, MAP_SHARED, NULL);
+         if (cpu != MAP_FAILED)
+            mem->debug.host_mapping = cpu;
+         else
+            vk_logw(VK_LOG_OBJS(_device),
+                    "failed to map VkMemory for dump or trace.\n");
       }
 
       pandecode_inject_mmap(device->debug.decode_ctx, mem->addr.dev,
