@@ -18,13 +18,14 @@
 #include "vn_instance.h"
 #include "vn_physical_device.h"
 #include "vn_queue.h"
+#include "vn_ring.h"
 
 /* device commands */
 
 static void
 vn_queue_fini(struct vn_queue *queue)
 {
-   VkDevice dev_handle = vk_device_to_handle(queue->base.base.base.device);
+   VkDevice dev_handle = vk_device_to_handle(queue->base.vk.base.device);
 
    if (queue->wait_fence != VK_NULL_HANDLE) {
       vn_DestroyFence(dev_handle, queue->wait_fence, NULL);
@@ -40,14 +41,25 @@ static VkResult
 vn_queue_init(struct vn_device *dev,
               struct vn_queue *queue,
               const VkDeviceQueueCreateInfo *queue_info,
-              uint32_t queue_index)
+              uint32_t queue_index,
+              struct vn_queue *shared_queue)
 {
    VkResult result =
       vn_queue_base_init(&queue->base, &dev->base, queue_info, queue_index);
    if (result != VK_SUCCESS)
       return result;
 
-   vn_cached_storage_init(&queue->storage, &dev->base.base.alloc);
+   vn_cached_storage_init(&queue->storage, &dev->base.vk.alloc);
+
+   if (dev->physical_device->emulate_second_queue ==
+          queue_info->queueFamilyIndex &&
+       shared_queue != NULL) {
+      assert(queue_index > 0);
+      queue->emulated = true;
+      queue->base.id = shared_queue->base.id;
+      queue->ring_idx = shared_queue->ring_idx;
+      return VK_SUCCESS;
+   }
 
    const int ring_idx = vn_instance_acquire_ring_idx(dev->instance);
    if (ring_idx < 0) {
@@ -79,7 +91,7 @@ static VkResult
 vn_device_init_queues(struct vn_device *dev,
                       const VkDeviceCreateInfo *create_info)
 {
-   const VkAllocationCallbacks *alloc = &dev->base.base.alloc;
+   const VkAllocationCallbacks *alloc = &dev->base.vk.alloc;
 
    uint32_t count = 0;
    for (uint32_t i = 0; i < create_info->queueCreateInfoCount; i++)
@@ -92,13 +104,15 @@ vn_device_init_queues(struct vn_device *dev,
       return VK_ERROR_OUT_OF_HOST_MEMORY;
 
    count = 0;
+   struct vn_queue *shared_queue = NULL;
    for (uint32_t i = 0; i < create_info->queueCreateInfoCount; i++) {
       VkResult result;
 
       const VkDeviceQueueCreateInfo *queue_info =
          &create_info->pQueueCreateInfos[i];
       for (uint32_t j = 0; j < queue_info->queueCount; j++) {
-         result = vn_queue_init(dev, &queues[count], queue_info, j);
+         result =
+            vn_queue_init(dev, &queues[count], queue_info, j, shared_queue);
          if (result != VK_SUCCESS) {
             for (uint32_t k = 0; k < count; k++)
                vn_queue_fini(&queues[k]);
@@ -107,7 +121,7 @@ vn_device_init_queues(struct vn_device *dev,
             return result;
          }
 
-         count++;
+         shared_queue = &queues[count++];
       }
    }
 
@@ -121,7 +135,7 @@ static bool
 vn_device_queue_family_init(struct vn_device *dev,
                             const VkDeviceCreateInfo *create_info)
 {
-   const VkAllocationCallbacks *alloc = &dev->base.base.alloc;
+   const VkAllocationCallbacks *alloc = &dev->base.vk.alloc;
    uint32_t *queue_families = NULL;
    uint32_t count = 0;
 
@@ -155,55 +169,7 @@ vn_device_queue_family_init(struct vn_device *dev,
 static inline void
 vn_device_queue_family_fini(struct vn_device *dev)
 {
-   vk_free(&dev->base.base.alloc, dev->queue_families);
-}
-
-static VkResult
-vn_device_memory_report_init(struct vn_device *dev,
-                             const VkDeviceCreateInfo *create_info)
-{
-   const struct vk_features *app_feats = &dev->base.base.enabled_features;
-   if (!app_feats->deviceMemoryReport)
-      return VK_SUCCESS;
-
-   uint32_t count = 0;
-   vk_foreach_struct_const(pnext, create_info->pNext) {
-      if (pnext->sType ==
-          VK_STRUCTURE_TYPE_DEVICE_DEVICE_MEMORY_REPORT_CREATE_INFO_EXT)
-         count++;
-   }
-
-   struct vn_device_memory_report *mem_reports = NULL;
-   if (count) {
-      mem_reports =
-         vk_alloc(&dev->base.base.alloc, sizeof(*mem_reports) * count,
-                  VN_DEFAULT_ALIGN, VK_SYSTEM_ALLOCATION_SCOPE_DEVICE);
-      if (!mem_reports)
-         return VK_ERROR_OUT_OF_HOST_MEMORY;
-   }
-
-   count = 0;
-   vk_foreach_struct_const(pnext, create_info->pNext) {
-      if (pnext->sType ==
-          VK_STRUCTURE_TYPE_DEVICE_DEVICE_MEMORY_REPORT_CREATE_INFO_EXT) {
-         const struct VkDeviceDeviceMemoryReportCreateInfoEXT *report =
-            (void *)pnext;
-         mem_reports[count].callback = report->pfnUserCallback;
-         mem_reports[count].data = report->pUserData;
-         count++;
-      }
-   }
-
-   dev->memory_report_count = count;
-   dev->memory_reports = mem_reports;
-
-   return VK_SUCCESS;
-}
-
-static inline void
-vn_device_memory_report_fini(struct vn_device *dev)
-{
-   vk_free(&dev->base.base.alloc, dev->memory_reports);
+   vk_free(&dev->base.vk.alloc, dev->queue_families);
 }
 
 static bool
@@ -258,7 +224,7 @@ vn_device_fix_create_info(const struct vn_device *dev,
 {
    const struct vn_physical_device *physical_dev = dev->physical_device;
    const struct vk_device_extension_table *app_exts =
-      &dev->base.base.enabled_extensions;
+      &dev->base.vk.enabled_extensions;
    /* extra_exts and block_exts must not overlap */
    const char *extra_exts[16];
    const char *block_exts[16];
@@ -293,6 +259,12 @@ vn_device_fix_create_info(const struct vn_device *dev,
             VK_KHR_SWAPCHAIN_MUTABLE_FORMAT_EXTENSION_NAME;
          block_exts[block_count++] =
             VK_KHR_INCREMENTAL_PRESENT_EXTENSION_NAME;
+         block_exts[block_count++] =
+            VK_EXT_SWAPCHAIN_MAINTENANCE_1_EXTENSION_NAME;
+         block_exts[block_count++] =
+            VK_EXT_HDR_METADATA_EXTENSION_NAME;
+         block_exts[block_count++] =
+            VK_EXT_DISPLAY_CONTROL_EXTENSION_NAME;
       }
 
       if (app_exts->ANDROID_native_buffer) {
@@ -342,6 +314,11 @@ vn_device_fix_create_info(const struct vn_device *dev,
          VK_EXT_EXTERNAL_MEMORY_ACQUIRE_UNMODIFIED_EXTENSION_NAME;
    }
 
+   if (app_exts->KHR_map_memory2) {
+      /* see vn_physical_device_get_native_extensions */
+      block_exts[block_count++] = VK_KHR_MAP_MEMORY_2_EXTENSION_NAME;
+   }
+
    if (app_exts->EXT_device_memory_report) {
       /* see vn_physical_device_get_native_extensions */
       block_exts[block_count++] = VK_EXT_DEVICE_MEMORY_REPORT_EXTENSION_NAME;
@@ -387,7 +364,7 @@ vn_device_feedback_pool_init(struct vn_device *dev)
     * events, which well covers the common scenarios. Pool can grow anyway.
     */
    static const uint32_t pool_size = 4096;
-   const VkAllocationCallbacks *alloc = &dev->base.base.alloc;
+   const VkAllocationCallbacks *alloc = &dev->base.vk.alloc;
 
    if (VN_PERF(NO_EVENT_FEEDBACK) && VN_PERF(NO_FENCE_FEEDBACK) &&
        VN_PERF(NO_SEMAPHORE_FEEDBACK))
@@ -419,7 +396,7 @@ vn_device_update_shader_cache_id(struct vn_device *dev)
     */
 #if !DETECT_OS_ANDROID && defined(ENABLE_SHADER_CACHE)
    const uint8_t *device_uuid =
-      dev->physical_device->base.base.properties.pipelineCacheUUID;
+      dev->physical_device->base.vk.properties.pipelineCacheUUID;
 
    char uuid[VK_UUID_SIZE * 2 + 1];
    mesa_bytes_to_hex(uuid, device_uuid, VK_UUID_SIZE);
@@ -470,8 +447,25 @@ vn_device_init(struct vn_device *dev,
    if (group && group->physicalDeviceCount)
       dev->device_mask = (1 << group->physicalDeviceCount) - 1;
 
+   VkDeviceCreateInfo final_create_info = *create_info;
+   STACK_ARRAY(VkDeviceQueueCreateInfo, queue_infos,
+               create_info->queueCreateInfoCount);
+   for (uint32_t i = 0; i < create_info->queueCreateInfoCount; i++) {
+      const VkDeviceQueueCreateInfo *queue_info =
+         &create_info->pQueueCreateInfos[i];
+      if (queue_info->queueFamilyIndex ==
+             physical_dev->emulate_second_queue &&
+          queue_info->queueCount == 2) {
+         typed_memcpy(queue_infos, create_info->pQueueCreateInfos,
+                      create_info->queueCreateInfoCount);
+         final_create_info.pQueueCreateInfos = queue_infos;
+         queue_infos[i].queueCount = 1;
+         break;
+      }
+   }
    result = vn_call_vkCreateDevice(dev->primary_ring, physical_dev_handle,
-                                   create_info, NULL, &dev_handle);
+                                   &final_create_info, NULL, &dev_handle);
+   STACK_ARRAY_FINISH(queue_infos);
 
    /* free the fixed extensions here since no longer needed below */
    if (create_info == &local_create_info)
@@ -480,13 +474,9 @@ vn_device_init(struct vn_device *dev,
    if (result != VK_SUCCESS)
       return result;
 
-   result = vn_device_memory_report_init(dev, create_info);
-   if (result != VK_SUCCESS)
-      goto out_destroy_device;
-
    if (!vn_device_queue_family_init(dev, create_info)) {
       result = VK_ERROR_OUT_OF_HOST_MEMORY;
-      goto out_memory_report_fini;
+      goto out_destroy_device;
    }
 
    result = vn_device_feedback_pool_init(dev);
@@ -520,11 +510,15 @@ out_feedback_pool_fini:
 out_queue_family_fini:
    vn_device_queue_family_fini(dev);
 
-out_memory_report_fini:
-   vn_device_memory_report_fini(dev);
-
 out_destroy_device:
-   vn_call_vkDestroyDevice(dev->primary_ring, dev_handle, NULL);
+   /* surpress -Wc23-extensions */
+   {
+      struct vn_ring_submit_command ring_submit;
+      vn_submit_vkDestroyDevice(dev->primary_ring, 0, dev_handle, NULL,
+                                &ring_submit);
+      if (ring_submit.ring_seqno_valid)
+         vn_ring_wait_seqno(dev->primary_ring, ring_submit.ring_seqno);
+   }
 
    return result;
 }
@@ -540,7 +534,7 @@ vn_CreateDevice(VkPhysicalDevice physicalDevice,
       vn_physical_device_from_handle(physicalDevice);
    struct vn_instance *instance = physical_dev->instance;
    const VkAllocationCallbacks *alloc =
-      pAllocator ? pAllocator : &instance->base.base.alloc;
+      pAllocator ? pAllocator : &instance->base.vk.alloc;
    struct vn_device *dev;
    VkResult result;
 
@@ -569,8 +563,8 @@ vn_CreateDevice(VkPhysicalDevice physicalDevice,
    }
 
    if (VN_DEBUG(LOG_CTX_INFO)) {
-      vn_log(instance, "%s", physical_dev->base.base.properties.deviceName);
-      vn_log(instance, "%s", physical_dev->base.base.properties.driverInfo);
+      vn_log(instance, "%s", physical_dev->base.vk.properties.deviceName);
+      vn_log(instance, "%s", physical_dev->base.vk.properties.driverInfo);
    }
 
    vn_tls_set_async_pipeline_create();
@@ -586,7 +580,7 @@ vn_DestroyDevice(VkDevice device, const VkAllocationCallbacks *pAllocator)
    VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    const VkAllocationCallbacks *alloc =
-      pAllocator ? pAllocator : &dev->base.base.alloc;
+      pAllocator ? pAllocator : &dev->base.vk.alloc;
 
    if (!dev)
       return;
@@ -603,16 +597,15 @@ vn_DestroyDevice(VkDevice device, const VkAllocationCallbacks *pAllocator)
 
    vn_device_queue_family_fini(dev);
 
-   vn_device_memory_report_fini(dev);
-
    vn_async_vkDestroyDevice(dev->primary_ring, device, NULL);
 
-   /* We must emit vn_call_vkDestroyDevice before releasing bound ring_idx.
-    * Otherwise, another thread might reuse their ring_idx while they
-    * are still bound to the queues in the renderer.
+   /* We must emit vkDestroyDevice before releasing bound ring_idx. Otherwise,
+    * another thread might reuse their ring_idx while they are still bound to
+    * the queues in the renderer.
     */
    for (uint32_t i = 0; i < dev->queue_count; i++) {
-      vn_instance_release_ring_idx(dev->instance, dev->queues[i].ring_idx);
+      if (!dev->queues[i].emulated)
+         vn_instance_release_ring_idx(dev->instance, dev->queues[i].ring_idx);
    }
 
    vk_free(alloc, dev->queues);
@@ -625,7 +618,7 @@ PFN_vkVoidFunction
 vn_GetDeviceProcAddr(VkDevice device, const char *pName)
 {
    struct vn_device *dev = vn_device_from_handle(device);
-   return vk_device_get_proc_addr(&dev->base.base, pName);
+   return vk_device_get_proc_addr(&dev->base.vk, pName);
 }
 
 void
@@ -645,10 +638,10 @@ vn_GetDeviceGroupPeerMemoryFeatures(
 }
 
 VkResult
-vn_GetCalibratedTimestampsEXT(
+vn_GetCalibratedTimestampsKHR(
    VkDevice device,
    uint32_t timestampCount,
-   const VkCalibratedTimestampInfoEXT *pTimestampInfos,
+   const VkCalibratedTimestampInfoKHR *pTimestampInfos,
    uint64_t *pTimestamps,
    uint64_t *pMaxDeviation)
 {
@@ -665,10 +658,10 @@ vn_GetCalibratedTimestampsEXT(
 
    for (domain = 0; domain < timestampCount; domain++) {
       switch (pTimestampInfos[domain].timeDomain) {
-      case VK_TIME_DOMAIN_DEVICE_EXT: {
+      case VK_TIME_DOMAIN_DEVICE_KHR: {
          uint64_t device_max_deviation = 0;
 
-         ret = vn_call_vkGetCalibratedTimestampsEXT(
+         ret = vn_call_vkGetCalibratedTimestampsKHR(
             dev->primary_ring, device, 1, &pTimestampInfos[domain],
             &pTimestamps[domain], &device_max_deviation);
 
@@ -678,12 +671,12 @@ vn_GetCalibratedTimestampsEXT(
          max_clock_period = MAX2(max_clock_period, device_max_deviation);
          break;
       }
-      case VK_TIME_DOMAIN_CLOCK_MONOTONIC_EXT:
+      case VK_TIME_DOMAIN_CLOCK_MONOTONIC_KHR:
          pTimestamps[domain] = vk_clock_gettime(CLOCK_MONOTONIC);
          max_clock_period = MAX2(max_clock_period, 1);
          break;
 #ifdef CLOCK_MONOTONIC_RAW
-      case VK_TIME_DOMAIN_CLOCK_MONOTONIC_RAW_EXT:
+      case VK_TIME_DOMAIN_CLOCK_MONOTONIC_RAW_KHR:
          pTimestamps[domain] = begin;
          break;
 #endif
