@@ -27,6 +27,6 @@
 #include <xcb/xcb.h>
 #include <inttypes.h>
 int x11_dri3_open(xcb_connection_t *conn, xcb_window_t root, uint32_t provider);
-bool x11_dri3_check_multibuffer(xcb_connection_t *c, bool *err, bool *explicit_modifiers);
+bool x11_dri3_has_multibuffer(xcb_connection_t *c);
 
 #endif
